//////////////////////////////////////////////////////////////////////////////
//
//    IMAGEFILEPREVIEW.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

#include "ImageFilePreview.h"


/////////////////////////////////////////////////////////////////////////////////////////////

ImageFilePreview::ImageFilePreview( QWidget *parent, QLabel* ImageLabelSource)
    : KPreviewWidgetBase( parent ),
      m_job( 0L )
{
    ImageLabel = ImageLabelSource;
    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), SLOT(showPreview()) );
}


/////////////////////////////////////////////////////////////////////////////////////////////

ImageFilePreview::~ImageFilePreview()
{
    if ( m_job )
        m_job->kill();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::showPreview( const KURL& url )
{
    showPreview( url, false );
}
  

/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::showPreview( const KURL &url, bool force )
{
    if ( url.isMalformed() )
        {
        clearPreview();
        return;
        }

    if ( url != currentURL || force )
        {
        clearPreview();
	      currentURL = url;

      	if ( autoMode || force )
	        {
          int w = ImageLabel->contentsRect().width() - 4;
          int h = ImageLabel->contentsRect().height() - 4;

          m_job =  createJob( url, w, h );

          connect( m_job, SIGNAL( result( KIO::Job * )),
                     this, SLOT( slotResult( KIO::Job * )));

          connect( m_job, SIGNAL( gotPreview( const KFileItem*,
                     const QPixmap& )), SLOT( gotPreview( const KFileItem*, const QPixmap& ) ));

          connect( m_job, SIGNAL( failed( const KFileItem* )),
                     this, SLOT( slotFailed( const KFileItem* ) ));
        	}
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::toggleAuto( bool a )
{
  autoMode = a;
  
  if ( autoMode )
    {
	  showPreview( currentURL, true );
    }
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::resizeEvent( QResizeEvent * )
{
    timer->start( 100, true ); // forces a new preview
}


/////////////////////////////////////////////////////////////////////////////////////////////

QSize ImageFilePreview::sizeHint() const
{
    return QSize( 20, 200 ); // otherwise it ends up huge???
}


/////////////////////////////////////////////////////////////////////////////////////////////

KIO::PreviewJob * ImageFilePreview::createJob( const KURL& url, int w, int h )
{
    KURL::List urls;
    urls.append( url );
    return KIO::filePreview( urls, w, h, 0, 0, true, false );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::gotPreview( const KFileItem* item, const QPixmap& pm )
{
    if ( item->url() == currentURL ) // should always be the case
        ImageLabel->setPixmap( pm );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::slotFailed( const KFileItem* item )
{
    if ( item->isDir() )
        ImageLabel->clear();
    else if ( item->url() == currentURL ) // should always be the case
        ImageLabel->setPixmap( SmallIcon( "file_broken", KIcon::SizeLarge, KIcon::DisabledState ));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::slotResult( KIO::Job *job )
{
    if ( job == m_job )
        m_job = 0L;
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::clearPreview()
{
  if ( m_job )
    {
        m_job->kill();
        m_job = 0L;
    }

  ImageLabel->clear();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ImageFilePreview::virtual_hook( int id, void* data )
{
  KPreviewWidgetBase::virtual_hook( id, data );
}

#include "ImageFilePreview.moc"
