//////////////////////////////////////////////////////////////////////////////
//
//    IMAGEFILEPREVIEW.H
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


#ifndef IMAGEFILEPREVIEW_H
#define IMAGEFILEPREVIEW_H

// Qt includes

#include <qpixmap.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>
#include <qtimer.h>

// KDElib includes

#include <kurl.h>
#include <kpreviewwidgetbase.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <kio/previewjob.h>


class QCheckBox;
class QPushButton;
class QLabel;
class QTimer;

class KFileDialog;
class KFileItem;
namespace KIO { class PreviewJob; }

class ImageFilePreview : public KPreviewWidgetBase
{
	Q_OBJECT

	public:
		ImageFilePreview(QWidget *parent, QLabel* ImageLabelSource);
		~ImageFilePreview();

		virtual QSize sizeHint() const;
 		virtual void showPreview( const KURL& url, bool force );
    virtual void clearPreview();
    
	public slots:
		virtual void showPreview(const KURL &url);
		
	protected slots:
    void showPreview() { showPreview( currentURL, true ); }
		void toggleAuto(bool);
		virtual void gotPreview( const KFileItem*, const QPixmap& );

	protected:
		virtual void resizeEvent(QResizeEvent *e);
		virtual KIO::PreviewJob * createJob( const KURL& url, int w, int h );

	private slots:
		void slotResult( KIO::Job * );
		virtual void slotFailed( const KFileItem* );

	private:
		bool              autoMode;
		KURL              currentURL;
		QTimer            *timer;
		QLabel            *infoLabel;
		QCheckBox         *autoPreview;
		QPushButton       *previewButton;
		KIO::PreviewJob   *m_job;
    QLabel            *ImageLabel;

  protected:
    virtual void virtual_hook( int id, void* data );

  private:
    class ImageFilePreviewPrivate;
    ImageFilePreviewPrivate *d;
};

#endif // IMAGEFILEPREVIEW_H
