//////////////////////////////////////////////////////////////////////////////
//
//    KIMG2MPG.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


// Local includes

#include "kimg2mpg.h"
#include "kimg2mpg.moc"



/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// CONSTRUCTOR /////////////////////////////////////////////////

KImg2mpgData::KImg2mpgData(QWidget * parent, const char * name) : KDialog( parent, name )
{
  Proc = 0L;
	Encoding = false;
	Abort = false;
  
  Icons = new KIconLoader( QString( "kimg2mpg" ) );
  TemporaryFolderName = "/tmp/";
  NoneLabel = i18n ("none");
  
  QVBoxLayout* ml = new QVBoxLayout( this, 10 );
  QHBoxLayout* h1 = new QHBoxLayout( ml );
  QVBoxLayout* v1 = new QVBoxLayout( h1 );
  h1->addSpacing( 5 );
  QGridLayout* g1 = new QGridLayout( v1, 3, 2 );

  // Video format selection...
          
  label1 = new QLabel(i18n("Video format and type :"), this);
  g1->addWidget( label1, 0, 0, AlignLeft );

  VideoFormatComboBox = new QComboBox( FALSE, this, "ComboBox_1" );
	g1->addWidget( VideoFormatComboBox, 0, 1, AlignLeft );
  VideoFormatComboBox->insertItem("XVCD");
	VideoFormatComboBox->insertItem("SVCD");
	VideoFormatComboBox->insertItem("VCD");
  VideoFormatComboBox->insertItem("DVD");
  QToolTip::add( VideoFormatComboBox, i18n( "This option specify the video format for your MPEG file." ) );
  QWhatsThis::add( VideoFormatComboBox, i18n( "This option specify the video format for your MPEG file. "
    "For a very good photographic restitution on TV screen, select 'XVCD' (it's the same DVD resolution), "
    "but some old DVD players can't read this format. "
    "'VCD'/'SVCD' are more compatible with the DVD player, but the are a medium photographic restitution on TV screen "
    "DVD is an experimental option." ) );
        
  // Video type selection...
  
	VideoTypeComboBox = new QComboBox( FALSE, this, "ComboBox_2" );
	g1->addWidget( VideoTypeComboBox, 0, 2, AlignLeft );
	VideoTypeComboBox->insertItem("PAL");
	VideoTypeComboBox->insertItem("NTSC");
	VideoTypeComboBox->insertItem("SECAM");
  QToolTip::add( VideoTypeComboBox, i18n( "This option specify the video type for your MPEG file." ) );
  QWhatsThis::add( VideoTypeComboBox, i18n( "This option specify the video type for your MPEG file. "
    "NTSC is an american TV standard, PAL/SECAM is European." ) );

	connect( VideoTypeComboBox, SIGNAL( activated(int ) ), this, SLOT( SlotPortfolioDurationChanged (int) ) );
      
  // Image duration...
  
  label3 = new QLabel(i18n("Image duration (seconds) :"), this);
  g1->addWidget( label3, 2, 0, AlignLeft );

	DurationImageSpinBox = new QSpinBox( 1, 999, 1 , this, "SpinBox_1" );
	g1->addWidget( DurationImageSpinBox, 2, 1, AlignLeft );
  QToolTip::add( DurationImageSpinBox, i18n( "This option specify the duration for each image in your MPEG file." ) );
  QWhatsThis::add( DurationImageSpinBox, i18n( "This option specify the duration for each image in your MPEG file. "
    "10 seconds is a good value for an images portfolio. "
    "Warning : you can have some problems with your DVD player if total MPEG duration is under 3 seconds." ) );

 	connect( DurationImageSpinBox, SIGNAL( valueChanged(int ) ), this, SLOT( SlotPortfolioDurationChanged (int) ) );
   
  // Transition between images selection...

  label4 = new QLabel(i18n("Transition speed between images :"), this);
  g1->addWidget( label4, 3, 0, AlignLeft );

	TransitionComboBox = new QComboBox( FALSE, this, "ComboBox_3" );
	g1->addWidget( TransitionComboBox, 3, 1, AlignLeft );
	TransitionComboBox->insertItem(NoneLabel);
	TransitionComboBox->insertItem("1");
	TransitionComboBox->insertItem("2");
	TransitionComboBox->insertItem("4");
	TransitionComboBox->insertItem("5");
	TransitionComboBox->insertItem("10");
	TransitionComboBox->insertItem("20");
  QToolTip::add( TransitionComboBox, i18n( "This option specify the transition speed between images in your MPEG file." ) );
  QWhatsThis::add( TransitionComboBox, i18n( "This option specify the transition speed between images in your MPEG file. "
    "'1' is a slow transition and '20' is very fast transition. "
    "'2' is a good value for an images portofio." ) );

	connect( TransitionComboBox, SIGNAL( activated(int ) ), this, SLOT( SlotPortfolioDurationChanged (int) ) );
     
  // Background color selection...

  label5 = new QLabel(i18n("Background color :"), this);
  g1->addWidget( label5, 4, 0, AlignLeft );

  QColor BackGroundCOLOR = QColor( 0, 0, 0 );                         // Black (default background color).
  BackgroundColorButton = new KColorButton( BackGroundCOLOR, this );
  g1->addWidget( BackgroundColorButton, 4, 1, AlignLeft );
  QToolTip::add( BackgroundColorButton, i18n( "You can select here the background color for your portfolio." ) );
  QWhatsThis::add( BackgroundColorButton, i18n( "You can select here the background color for your portfolio. "
    "This color is used for adapt the image size with the TV screen size. "
    "Black color is a good value for this. " ));
         
  // MPEG output filename selection...

  MPEGOutputFilename = new QGroupBox( 2, Qt::Horizontal, i18n( "MPEG output filename" ), this);
  v1->addWidget( MPEGOutputFilename );

  MPEGOutputEDITFilename = new KLineEdit( MPEGOutputFilename );
	MPEGOutputEDITFilename->setMinimumWidth( 300 );
  MPEGOutputBUTTONFilename = new QPushButton( MPEGOutputFilename );
  MPEGOutputBUTTONFilename->setPixmap( LoadIcon( QString( "fileopen" ), KIcon::Toolbar ) );
  connect( MPEGOutputBUTTONFilename, SIGNAL( clicked() ), this, SLOT( slotMPEGFilenameDialog() ) );
  QToolTip::add( MPEGOutputEDITFilename, i18n( "You can specify here the output MPEG file name." ) );
  QWhatsThis::add( MPEGOutputEDITFilename, i18n( "You can specify here the output MPEG file name. "
    "Warning : MPEG files are very big files (when you use many images in your portfolio). "
    "Select a folder with a big space disk. "
    "Note : Kimg2mpg use also '/tmp' folder for create some temporary files. "
    "Check your spacedisk in this folder.") );

  // Audio input filename selection...

 	AudioInputFilename = new QGroupBox( 2, Qt::Horizontal, i18n( "Audio input filename" ), this);
  v1->addWidget( AudioInputFilename );

  AudioInputEDITFilename = new KLineEdit( AudioInputFilename );
	AudioInputEDITFilename->setMinimumWidth( 300 );
  AudioInputBUTTONFilename = new QPushButton( AudioInputFilename );
  AudioInputBUTTONFilename->setPixmap( LoadIcon( QString( "fileopen" ), KIcon::Toolbar ) );
  connect( AudioInputBUTTONFilename, SIGNAL( clicked() ), this, SLOT( slotAudioFilenameDialog() ) );
  QToolTip::add( AudioInputEDITFilename, i18n( "You can specify here the input audio file name." ) );
  QWhatsThis::add( AudioInputEDITFilename, i18n( "You can specify here the input audio file name. "
    "This audio file name will be multiplexed with the portfolio video flux. "
    "Warning : if the audio duration is too long, it will be truncated." ) );
  
  // Images files list and the control buttons...  

  ImagesFilesGroup = new QGroupBox(	3, Qt::Horizontal, i18n( "Images files portfolio" ), this );
  v1->addWidget( ImagesFilesGroup );
                                                                                          
	ImagesFilesListBox = new KListBox( ImagesFilesGroup );
  ImagesFilesListBox->setSelectionMode (QListBox::Extended);
  ImagesFilesListBox->setMinimumWidth( 300 );
  QToolTip::add( ImagesFilesListBox, i18n( "This is the list of images files for your portfolio." ) );
  QWhatsThis::add( ImagesFilesListBox, i18n( "This is the list of images files for your portfolio. "
    "The portfolio first image is on the top, the last image is on the bottom. " ) );

  connect( ImagesFilesListBox, SIGNAL( highlighted(int) ), this, SLOT( slotImagesFilesSelected(int) ) );

  ImagesFilesButtonBox = new KButtonBox( ImagesFilesGroup, Vertical );
	ImagesFilesButtonAdd = ImagesFilesButtonBox->addButton( i18n( "&Add..." ) );
  QToolTip::add( ImagesFilesButtonAdd, i18n( "Add some images files on the portfolio list." ) );
	ImagesFilesButtonDelete = ImagesFilesButtonBox->addButton( i18n( "&Delete" ) );
  QToolTip::add( ImagesFilesButtonDelete, i18n( "Remove some images files on the portfolio list." ) );
  ImagesFilesButtonUp = ImagesFilesButtonBox->addButton( i18n( "&Up" ) );
  QToolTip::add( ImagesFilesButtonUp, i18n( "Moving up the current image on the portfolio list." ) );
	ImagesFilesButtonDown = ImagesFilesButtonBox->addButton( i18n( "D&own" ) );
  QToolTip::add( ImagesFilesButtonDown, i18n( "Moving down the current image on the portfolio list." ) );
  ImagesFilesButtonBox->layout();

	connect( ImagesFilesButtonAdd, SIGNAL( clicked() ), this, SLOT( slotImagesFilesButtonAdd() ) );
	connect( ImagesFilesButtonDelete, SIGNAL( clicked() ), this, SLOT( slotImagesFilesButtonDelete() ) );
  connect( ImagesFilesButtonUp, SIGNAL( clicked() ), this, SLOT( slotImagesFilesButtonUp() ) );
	connect( ImagesFilesButtonDown, SIGNAL( clicked() ), this, SLOT( slotImagesFilesButtonDown() ) );

  ImageLabel = new QLabel( ImagesFilesGroup );
  ImageLabel->setMinimumWidth( 120 );
  ImageLabel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  ImageLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
  ImageLabel->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred ) );
  QToolTip::add( ImageLabel, i18n( "Preview the current selected image." ) );
  
  label6 = new QLabel(ImagesFilesGroup);
  QToolTip::add( label6, i18n( "Total number of images in the portfolio and sequence duration." ) );
  label7 = new QLabel(ImagesFilesGroup);
  QToolTip::add( label7, i18n( "Current selected image in the portfolio list." ) );
	
  PreviewImage = new ImageFilePreview(NULL, ImageLabel);

  // Encode push button...      

  QVBoxLayout* v3 = new QVBoxLayout( h1 );
	Encodebutton = new QPushButton( this, "PushButton_3" );
	Encodebutton->setText(i18n( "&Encode") );
	Encodebutton->setAutoRepeat( FALSE );
  QToolTip::add( Encodebutton, i18n( "Start the portfolio MPEG encoding." ) );
  QWhatsThis::add( Encodebutton, i18n( "Start the portfolio MPEG encoding. "
    "The program use the 'img2mpg' bash script from 'MjpegTools' package. " ) );

  connect(Encodebutton, SIGNAL(clicked()), this, SLOT(slotEncode()));
  v3->addWidget( Encodebutton );
  v3->addStretch( 1 );

	// Setup the Help Menu...

  helpMenu = new KHelpMenu(this, KGlobal::instance()->aboutData(), false);
	helpbutton = new QPushButton( this, "PushButton_4" );
	helpbutton->setText(i18n( "&Help") );
	helpbutton->setAutoRepeat( FALSE );
	helpbutton->setPopup(helpMenu->menu());
	v3->addWidget( helpbutton );

  // Quit push button...

  quitbutton = new QPushButton( this, "PushButton_1" );
	quitbutton->setText(i18n( "&Quit") );
	quitbutton->setAutoRepeat( FALSE );
	connect(quitbutton, SIGNAL(clicked()), this, SLOT(slotQuit()));
  v3->addWidget( quitbutton );
  QToolTip::add( quitbutton, i18n( "Abort the current encoding and exit." ) );
  
  // Process messages frame...

	frame = new QLabel( this, "NewsWindow" );
	frame->setMinimumHeight( 25 );
	frame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	frame->setAlignment(AlignCenter|WordBreak|ExpandTabs);
  ml->addWidget( frame );
  QToolTip::add( frame, i18n( "Current encoding task." ) );

  // Progress bar...

	progress = new KProgress( this, "Progress" );
  progress->setMinimumHeight( 25 );
  ml->addWidget( progress );
  QToolTip::add( progress, i18n( "Encoding progress bar." ) );

  readSettings();
  findExecutables();

  int maxW = QMAX( VideoFormatComboBox->sizeHint().width(), VideoTypeComboBox->sizeHint().width() );
  VideoFormatComboBox->setMinimumWidth( maxW );
  VideoTypeComboBox->setMinimumWidth( maxW );
  TransitionComboBox->setMinimumWidth( maxW );
}


/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// DESTRUCTOR //////////////////////////////////////////////////

KImg2mpgData::~KImg2mpgData()
{
  delete Icons;
}


/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// SLOTS FONCTIONS /////////////////////////////////////////////

void KImg2mpgData::SlotPortfolioDurationChanged ( int )
{
  ShowNumberImages( ImagesFilesListBox->count() );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotMPEGFilenameDialog( void )
{
	QString temp;

	temp = KFileDialog::getOpenFileName(	QString::null,
																			  QString( "*.mpg" ),
																			  this,
																			  QString( i18n("Select MPEG output file...")) );
	if( temp.isEmpty() )
    return;

	MPEGOutputEDITFilename->setText( temp );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotAudioFilenameDialog( void )
{
	QString temp;

	temp = KFileDialog::getOpenFileName(	QString::null,
																			  QString( "*.wav *.mp2" ),
																			  this,
																			  QString( i18n("Select audio input file...")) );
	if( temp.isEmpty() )
    return;

	AudioInputEDITFilename->setText( temp );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotImagesFilesButtonAdd( void )
{
  QStringList ImageFilesList;

  ImageFilesList = KFileDialog::getOpenFileNames( QString::null,
                                                  KImageIO::pattern( KImageIO::Reading ),
                                                  this );

  if( ImageFilesList.isEmpty() )
    return;

  ImagesFilesListBox->insertStringList(ImageFilesList, ImagesFilesListBox->currentItem() );
  ShowNumberImages( ImagesFilesListBox->count() );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotImagesFilesButtonDelete( void )
{
  int First=FALSE;
  int IndexFirst=0;

  for (uint i=0; i < ImagesFilesListBox->count() ; i++)
    {
    if (ImagesFilesListBox->isSelected(i))
      {
      if (First == FALSE)
        {
        First=TRUE;
        IndexFirst=i;
        }
      ImagesFilesListBox->setCurrentItem(i);
      ImagesFilesListBox->removeItem(i);
      i=0;
      }
    }

  ImagesFilesListBox->setSelected(IndexFirst, TRUE);
  ShowNumberImages( ImagesFilesListBox->count() );

  if (ImagesFilesListBox->count() == 0)
    {
    label7->setText("");
    PreviewImage->clearPreview();
    return;
    }
}

                                   
/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotImagesFilesButtonUp( void )
{
  int Cpt=0;

  for (uint i=0; i < ImagesFilesListBox->count() ; i++)
    if (ImagesFilesListBox->isSelected(i))
      Cpt++;

  if  (Cpt == 0)
    return;

  if  (Cpt > 1)
    {
    KMessageBox::error(this, i18n("You can just moving up one selected image file name !"));
    return;
    }
          
  int Index=ImagesFilesListBox->currentItem();
  QString Name=ImagesFilesListBox->item(Index)->text();
  ImagesFilesListBox->removeItem(Index);
  ImagesFilesListBox->insertItem(Name, Index-1);
  ImagesFilesListBox->setSelected(Index-1, TRUE);
  ImagesFilesListBox->setCurrentItem(Index-1);
}

/////////////////////////////////////////////////////////////////////////////////////////////


void KImg2mpgData::slotImagesFilesSelected( int Index )
{
  KURL url;
  QString IdemIndexed = "file:/";

  if (Index == -1)      // 2 signals will be sent by the KListBox : the focus (Index=-1) and the
    return;             // highlighted image on the list.
  
  IdemIndexed.append (ImagesFilesListBox->item(Index)->text());
  url=IdemIndexed;
  PreviewImage->showPreview (url, true);
  label7->setText(i18n("Image no. %1").arg(Index+1));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotImagesFilesButtonDown( void )
{
  int Cpt=0;

  for (uint i=0; i < ImagesFilesListBox->count() ; i++)
    if (ImagesFilesListBox->isSelected(i))
      Cpt++;

  if  (Cpt == 0)
    return;
      
  if  (Cpt > 1)
    {
    KMessageBox::error(this, i18n("You can just moving down one selected image file name !"));
    return;
    }

  int Index=ImagesFilesListBox->currentItem();
  QString Name=ImagesFilesListBox->item(Index)->text();
  ImagesFilesListBox->removeItem(Index);
  ImagesFilesListBox->insertItem(Name, Index+1);
  ImagesFilesListBox->setSelected(Index+1, TRUE);
  ImagesFilesListBox->setCurrentItem(Index+1);
} 


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotQuit()
{
  if (Encoding)
    {
    int Ret=KMessageBox::questionYesNo(this, i18n("An encoding process is active !\nDo you really abort this process and exit ?"));
    if (Ret == KMessageBox::Yes)
      {
      Abort = true;
      reset();
      RemoveTmpFiles();
      }
    else
      return;
    }
                 
  writeSettings();
  kapp->quit();
  delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::slotEncode( void )
{
  Proc = new KProcess;
  QColor BackGroundColor;
  QString HexColor;
  QString Temp;
  QString OutputFileName, InputAudioFileName;
  bool ResultOk;

  if (Encoding)
    {
    int Ret=KMessageBox::questionYesNo(this, i18n("Do you really abort this encoding process ?\n\nWarning : all works will be lost ! "));
    if (Ret == KMessageBox::Yes)
      {
      Abort = true;
      reset();
      }
    return;
    }

  DebugOuputMessages="";  
  DurationTime.start();
  InputAudioFileName = AudioInputEDITFilename->text();  
  OutputFileName = MPEGOutputEDITFilename->text();
    
  if (OutputFileName.isEmpty())
    {
    KMessageBox::error(this, i18n("You must specify an MPEG output file name !"));
    return;
    }

  if (ImagesFilesListBox->count() == 0)
    {
    KMessageBox::error(this, i18n("You must specify some input images files in the portfolio !"));
    return;
    }

  if (QFile::exists(OutputFileName))
    {
    int Ret=KMessageBox::questionYesNo(this, i18n("The output MPEG file '%1' already exist !\n"
                                                  "Do you want overwrite this file ?").arg(OutputFileName));
    if (Ret == KMessageBox::No)
      return;
    }
  ImagesFilesListBox->clearSelection();
  Encodebutton->setText(i18n("A&bort"));
  QToolTip::add( Encodebutton, i18n( "Abort the portfolio MPEG encoding." ) );
  QWhatsThis::add( Encodebutton, i18n( "Abort the portfolio MPEG encoding. "
    "Warning : all works will be lost..." ) );

  VideoFormatComboBox->setEnabled(false);
  VideoTypeComboBox->setEnabled(false);
  DurationImageSpinBox->setEnabled(false);
  TransitionComboBox->setEnabled(false);
  MPEGOutputEDITFilename->setEnabled(false);
	MPEGOutputBUTTONFilename->setEnabled(false);
  BackgroundColorButton->setEnabled(false);
  AudioInputEDITFilename->setEnabled(false);
  AudioInputBUTTONFilename->setEnabled(false);
  ImagesFilesListBox->setEnabled(false);
  ImagesFilesButtonBox->setEnabled(false);
                                                                                                                                                    
  Abort = false;
  Encoding = true;

  progress->setRange(0, 100);
  progress->setValue(0);

  BackGroundColor=BackgroundColorButton->color();
  HexColor="";
  Temp.setNum (BackGroundColor.red(), 16);
  HexColor=HexColor+Temp;
  Temp.setNum (BackGroundColor.green(), 16);
  HexColor=HexColor+Temp;
  Temp.setNum (BackGroundColor.blue(), 16);
  HexColor=HexColor+Temp;

  CommandLine=i18n("THE COMMAND LINE IS :\n\n");                // This is for debuging output in debug dialog box.
  CommandLine=CommandLine + "img2mpg --with-gui "; 
                      
  *Proc << "img2mpg";                                           // The script...
  *Proc << "--with-gui";                                        // Running with the GUI.
  *Proc << "-f" << VideoFormatComboBox->currentText();          // Video format option.
  CommandLine=CommandLine+" -f "+ VideoFormatComboBox->currentText();

  *Proc << "-n" << VideoTypeComboBox->currentText();            // Video type option.
  CommandLine=CommandLine+" -n "+ VideoTypeComboBox->currentText();
  
  *Proc << "-d" << DurationImageSpinBox->text();                // Image duration.
  CommandLine=CommandLine+" -d "+ DurationImageSpinBox->text();

  TransitionComboBox->currentText().toInt(&ResultOk);
  if (ResultOk == TRUE)
      {
      *Proc << "-t" << TransitionComboBox->currentText();       // Transitions duration.
      CommandLine=CommandLine+" -t "+ TransitionComboBox->currentText();
      }
      
  *Proc << "-c" << HexColor;                                    // Background color.
  CommandLine=CommandLine+" -c "+ HexColor;

  int IndexTmp=0;
  bool Ret;
  do
    {
    IndexTmp++;
    QString Tmp;
    QDir FolderTmp;
    FolderTemporaryIndex=TemporaryFolderName;
    Tmp.setNum (IndexTmp, 10);
    FolderTemporaryIndex.append (Tmp);
    FolderTemporaryIndex.append (".kimg2mpg.tmp");
    Ret = FolderTmp.mkdir (FolderTemporaryIndex, TRUE);
    }
  while (Ret == FALSE);

  *Proc << "-T" << FolderTemporaryIndex;                     // Temporary folder.
  CommandLine=CommandLine+" -T "+ FolderTemporaryIndex;

  if ( InputAudioFileName.isEmpty() == FALSE )
    {
    if ( InputAudioFileName.findRev(".wav", -1, FALSE) == -1 )
      {
      *Proc << "-a" << InputAudioFileName;                      // Input MP2 audio file name.
      CommandLine=CommandLine+" -a "+ InputAudioFileName;
      }
    else
      {
      *Proc << "-w" << InputAudioFileName;                      // Input WAV audio file name.
      CommandLine=CommandLine+" -w "+ InputAudioFileName;
      }
    }

  *Proc << "-o" << OutputFileName;                              // Output MPEG file.
  CommandLine=CommandLine+" -o "+ OutputFileName;

  *Proc << "-i";                                                // Input images option.
  CommandLine=CommandLine+" -i ";

  for (uint i=0; i < ImagesFilesListBox->count() ; i++)
    {
    QString FileName="";
    FileName.append (ImagesFilesListBox->item(i)->text());      // Input images files.
    *Proc << FileName;
    CommandLine=CommandLine+" " + FileName + " ";
    }

  connect(Proc, SIGNAL(processExited(KProcess *)),this,
          SLOT(EncodeDone(KProcess*)));

  connect(Proc, SIGNAL(receivedStdout(KProcess *,char*,int)),this,
	        SLOT(readStdout(KProcess*,char*,int)));

  connect(Proc, SIGNAL(receivedStderr(KProcess *,char*,int)),this,
      	  SLOT(readStderr(KProcess*,char*,int)));

  bool result = Proc->start(KProcess::NotifyOnExit , KProcess::All);

  if(!result)
    {
    QString str = i18n("Cannot start 'img2mpg' bash script from MjpegTools package : fork failed.");
    KMessageBox::error(this, str);
    reset();
    }

  Img2mpgPidNum=Proc->pid();
}


/////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////  FONCTIONS ///////////////////////////////////////////////////

void KImg2mpgData::ShowNumberImages( int Number )
{
  QTime TotalDuration (0, 0, 0);
  bool ResultOk;
  int TransitionDuration;

  int DurationImage = DurationImageSpinBox->text().toInt();
  int TransitionSpeed = TransitionComboBox->currentText().toInt(&ResultOk);

  if (ResultOk == FALSE)
     TransitionDuration = 0;
  else
    {
    QString VidFormat=VideoTypeComboBox->currentText();

    if (VidFormat == "NTSC")
      TransitionDuration = (int)((((float)100 / (float)TransitionSpeed) / (float)30) * (float)1000) ;     // in ms
    else
      TransitionDuration = (int)((((float)100 / (float)TransitionSpeed) / (float)25) * (float)1000) ;     // in ms
    }

  TotalDuration = TotalDuration.addSecs(Number*DurationImage);
  TotalDuration = TotalDuration.addMSecs((Number+1)*TransitionDuration);

  if ( Number < 2)
    label6->setText(i18n("%1 image [%2]").arg(Number).arg(TotalDuration.toString()));
  else
    label6->setText(i18n("%1 images [%2]").arg(Number).arg(TotalDuration.toString()));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::readStdout(KProcess *, char *buffer, int buflen)
{
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::readStderr(KProcess *, char *buffer, int buflen)
{
  QString NewString2;
  int ImgNum;
  EncodeString = QString::fromLocal8Bit(buffer, buflen);

  QString BufferTmp = buffer;
//  std::cout << BufferTmp.left(buflen);

  DebugOuputMessages.append(BufferTmp.left(buflen));

  if (EncodeString.contains("Images encoding (%) :"))
    {
    int pos1 = EncodeString.find(':');

    if (pos1 != -1)
      {
      QString newstring = EncodeString.mid(pos1+1, 4);
      progress->setValue(newstring.toUInt());
      }

    int pos2 = EncodeString.find('[');

    if (pos2 != -1)
      {
      NewString2 = EncodeString.mid(pos2+1, 4);
      ImgNum = NewString2.toInt();

      if (ImgNum == 0)
        ImgNum = 1;

      frame->setText(i18n("Encoding images files [%1/%2]...").arg(ImgNum).arg(ImagesFilesListBox->count()));

      if (ImgNum > 1)
        ImagesFilesListBox->setSelected(ImgNum-2, FALSE);

      ImagesFilesListBox->setSelected(ImgNum-1, TRUE);
      ImagesFilesListBox->setCurrentItem(ImgNum-1);
      }
    }
  else
    {
    // Print on the GUI the actual 'img2mpg' process.

    if (EncodeString.contains("Initialising..."))
      frame->setText(i18n("Initialising..."));

    if (EncodeString.contains("Merging MPEG flux..."))
      {
      frame->setText(i18n("Merging MPEG flux..."));
      progress->setValue(100);
      }

    if (EncodeString.contains("Encoding audio file..."))
      {
      frame->setText(i18n("Encoding audio file..."));
      progress->setValue(100);
      }

    // Or errors detections...

    if (EncodeString.contains("cat:") ||
        EncodeString.contains("ERROR:") ||
        EncodeString.contains("Broken pipe") ||
        EncodeString.contains("No such file or directory"))
      {

      Abort = true;
      reset();
      int Ret=KMessageBox::warningYesNo(this,
                                 i18n("The 'img2mpg' script has returned an error during the MPEG encoding !\n"
                                      "The processus has been aborted !\n\n"
                                      "Send a mail to the author..."),
                                 i18n("'img2mpg' script execution problem"),
                                 i18n("OK"),
                                 i18n("Show debugging output"));
      if (Ret == KMessageBox::No)
        {
        DebuggingDialog = new KShowDebuggingOutput(DebugOuputMessages, CommandLine, i18n ("\nEXIT STATUS : error during encoding process !!!"), this);
        DebuggingDialog->exec();
        }
      }
    }
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::EncodeDone(KProcess*)
{
  reset();

  int msecDur=DurationTime.elapsed();
  EncodingDuration.setHMS(0, 0, 0);
  QTime Duration=EncodingDuration.addMSecs (msecDur);
  QString Encoding=Duration.toString("hh:mm:ss");

  if ( Abort == false )
    {
    frame->setText(i18n("Encoding terminated..."));
    int Ret=KMessageBox::warningYesNo(this,
                         i18n("The encoding process is terminated...\n\n"
                         "Encoding duration : %1").arg(Encoding),
                         i18n("'img2mpg' script execution terminated"),
                         i18n("OK"),
                         i18n("Show processus messages"));
    if (Ret == KMessageBox::No)
      {
      DebuggingDialog = new KShowDebuggingOutput(DebugOuputMessages, CommandLine, i18n ("\nEXIT STATUS : encoding process finished succefully !!!") ,this);
      DebuggingDialog->exec();
      }
    }
  else
    {
    frame->setText(i18n("Encoding aborted..."));
    int Ret=KMessageBox::warningYesNo(this,
                         i18n("The encoding process have been aborted...\n\n"
                         "Encoding duration : %1").arg(Encoding),
                         i18n("'img2mpg' script execution aborted"),
                         i18n("OK"),
                         i18n("Show processus messages"));
    if (Ret == KMessageBox::No)
      {
      DebuggingDialog = new KShowDebuggingOutput(DebugOuputMessages, CommandLine, i18n ("\nEXIT STATUS : encoding process aborted by user !!!"), this);
      DebuggingDialog->exec();
      }
    }

  RemoveTmpFiles();
}


/////////////////////////////////////////////////////////////////////////////////////////////

QPixmap KImg2mpgData::LoadIcon( QString Name, int Group )
{
  return Icons->loadIcon( Name, (KIcon::Group)Group );
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::closeEvent(QCloseEvent*)
{
  slotQuit();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::show()
{
  setCaption(i18n("KDE MPEG portfolio images creator"));
  KDialog::show();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::findExecutables( void )
{
  QString path = getenv("PATH");
  path.append(":/usr/bin:/usr/share/bin:/usr/local/bin");

  img2mpg   = KGlobal::dirs()->findExe("img2mpg", path);
  montage   = KGlobal::dirs()->findExe("montage", path);
  composite = KGlobal::dirs()->findExe("composite", path);
  convert   = KGlobal::dirs()->findExe("convert", path);
  identify  = KGlobal::dirs()->findExe("identify", path);
  ppmtoy4m  = KGlobal::dirs()->findExe("ppmtoy4m", path);
  yuvscaler = KGlobal::dirs()->findExe("yuvscaler", path);
  mpeg2enc  = KGlobal::dirs()->findExe("mpeg2enc", path);
  mp2enc    = KGlobal::dirs()->findExe("mp2enc", path);
  mplex     = KGlobal::dirs()->findExe("mplex", path);

  QString str = "";
  if (img2mpg.isEmpty())
    str = i18n("Cannot find 'img2mpg' script from MjegTools package ! Please install it...");

  if (montage.isEmpty())
    str = i18n("Cannot find 'montage' binary program from ImageMagick package ! Please install it...");

  if (composite.isEmpty())
    str = i18n("Cannot find 'composite' binary program from ImageMagick package ! Please install it...");

  if (convert.isEmpty())
    str = i18n("Cannot find 'convert' binary program from ImageMagick package ! Please install it...");

  if (identify.isEmpty())
    str = i18n("Cannot find 'identify' binary program from ImageMagick package ! Please install it...");

  if (ppmtoy4m.isEmpty())
    str = i18n("Cannot find 'ppmtoy4m' binary program from MjegTools package ! Please install it...");

  if (yuvscaler.isEmpty())
    str = i18n("Cannot find 'yuvscaler' binary program from MjegTools package ! Please install it...");

  if (mpeg2enc.isEmpty())
    str = i18n("Cannot find 'mpeg2enc' binary program from MjegTools package ! Please install it...");

  if (mp2enc.isEmpty())
    str = i18n("Cannot find 'mp2enc' binary program from MjegTools package ! Please install it...");

  if (mplex.isEmpty())
    str = i18n("Cannot find 'mplex' binary program from MjegTools package ! Please install it...");

  if (str != "")
    {
    Encodebutton->setEnabled(FALSE);
    str=str+i18n("\nPlease, visit this URLs :"
            "\nImageMagick package : http://www.imagemagick.org/"
            "\nMjpegTools package  : http://mjpeg.sourceforge.net/"
            "\n\n'img2mpg' bash script is a part of MjpegTools release >= 1.6.2."
            "\nIf you have an old MjpegTools release on your system, you can"
            "\ndownload this script from this URL :"
            "\nhttp://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/mjpeg/mjpeg_play/scripts/"
            "\nInstall it in your system binary folder (usually '/usr/bin').");
    KMessageBox::error(this, str);
    }
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::reset()
  {
  Encoding = false;

  if (Proc)
    {
    QString Temp;
    Temp=Temp.setNum (Img2mpgPidNum);
    KProcess* KillProc;
    KillProc= new KProcess;
    *KillProc << "kill" << "-9" << Temp;
    KillProc->start();
    }

  delete Proc;
  Proc = 0L;

  progress->setValue(0);
  frame->clear();

  Encodebutton->setText(i18n("&Encode"));
  VideoFormatComboBox->setEnabled(true);
  VideoTypeComboBox->setEnabled(true);
  DurationImageSpinBox->setEnabled(true);
  TransitionComboBox->setEnabled(true);
  MPEGOutputEDITFilename->setEnabled(true);
	MPEGOutputBUTTONFilename->setEnabled(true);
  BackgroundColorButton->setEnabled(true);
  AudioInputEDITFilename->setEnabled(true);
  AudioInputBUTTONFilename->setEnabled(true);
  ImagesFilesListBox->setEnabled(true);
  ImagesFilesButtonBox->setEnabled(true);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::writeSettings()
{
  config = kapp->config();
	config->setGroup("GeneralData");

  VideoFormatConfig = VideoFormatComboBox->currentText();
  config->writeEntry("VideoFormat", VideoFormatConfig);

  VideoTypeConfig = VideoTypeComboBox->currentText();
  config->writeEntry("VideoType", VideoTypeConfig);

  ImageDurationConfig = DurationImageSpinBox->text();
  config->writeEntry("ImageDuration", ImageDurationConfig);

  TransitionSpeedConfig = TransitionComboBox->currentText();
  config->writeEntry("TransitionSpeed", TransitionSpeedConfig);

  BackgroundColorConfig = BackgroundColorButton->color();
  config->writeEntry("BackgroundColor", BackgroundColorConfig);

  AudioInputFileConfig = AudioInputEDITFilename->text();
  config->writeEntry("AudioInputFile", AudioInputFileConfig);

  MPEGOutputFileConfig = MPEGOutputEDITFilename->text();;
  config->writeEntry("MPEGOutputFile", MPEGOutputFileConfig);

  config->sync();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::readSettings()
{
  config = kapp->config();
	config->setGroup("GeneralData");

  VideoFormatConfig = config->readEntry("VideoFormat", "XVCD");
  for (int i = 0 ; i < VideoFormatComboBox->count(); i++)
    if ( (QString) VideoFormatComboBox->text(i) == VideoFormatConfig)
      VideoFormatComboBox->setCurrentItem(i);

  VideoTypeConfig = config->readEntry("VideoType", "PAL");
  for (int i = 0 ; i < VideoTypeComboBox->count(); i++)
    if ( (QString) VideoTypeComboBox->text(i) == VideoTypeConfig)
      VideoTypeComboBox->setCurrentItem(i);

  ImageDurationConfig = config->readEntry("ImageDuration", "10");
  DurationImageSpinBox->setValue(ImageDurationConfig.toInt());

  TransitionSpeedConfig = config->readEntry("TransitionSpeed", NoneLabel);
  for (int i = 0 ; i < TransitionComboBox->count(); i++)
    if ( (QString) TransitionComboBox->text(i) == TransitionSpeedConfig)
      TransitionComboBox->setCurrentItem(i);

  QColor *ColorB = new QColor( 0, 0, 0 );
  BackgroundColorConfig = config->readColorEntry("BackgroundColor", ColorB);
  BackgroundColorButton->setColor(BackgroundColorConfig);

  AudioInputFileConfig = config->readEntry("AudioInputFile", "");
  AudioInputEDITFilename->setText(AudioInputFileConfig);

  MPEGOutputFileConfig = config->readEntry("MPEGOutputFile", "/tmp/output.mpg");
  MPEGOutputEDITFilename->setText(MPEGOutputFileConfig);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void KImg2mpgData::RemoveTmpFiles(void)
{
  if ( FolderTemporaryIndex.isEmpty() == FALSE )
    {
    QStringList FilesList;
    QDir TmpFolder(FolderTemporaryIndex);
    QString FullFileName;

    FilesList = TmpFolder.entryList();

    // Remove all files in the tmp folder.

    for ( QStringList::Iterator it = FilesList.begin(); it != FilesList.end(); ++it )
      {
      if (*it != "." && *it != "..")
        {
        FullFileName=FolderTemporaryIndex + "/" + *it;
        if (TmpFolder.remove (FullFileName) == FALSE)
          KMessageBox::error(this, i18n("Cannot remove temporary file :\n'%1'").arg(FullFileName));
        }
      }

    // Remove tmp folder.

    FullFileName=FolderTemporaryIndex + "/";
    if (TmpFolder.rmdir(FullFileName) ==FALSE )
      KMessageBox::error(this, i18n("Cannot remove temporary folder :\n'%1'").arg(FullFileName));
    }
}
