//////////////////////////////////////////////////////////////////////////////
//
//    KIMG2MPG.H
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>   
//    
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


#ifndef KImg2mpgData_included
#define KImg2mpgData_included

// Qt includes

#include <qwidget.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qdialog.h>
#include <qprogressbar.h>
#include <qtimer.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qheader.h>
#include <qtooltip.h>
#include <qwhatsthis.h> 
#include <qprocess.h>
#include <qdatetime.h>
#include <qlayout.h>

// C++ includes
  
#include <iostream>

// KDElib includes

#include <kprogress.h>
#include <kprocess.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kdialog.h>
#include <kfiledialog.h>
#include <kcolordialog.h>
#include <khelpmenu.h>
#include <kpopupmenu.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <kcolorbtn.h>
#include <klistbox.h>
#include <kbuttonbox.h>
#include <kurl.h>
#include <kimageio.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kstandarddirs.h>

// Local includes

#include "ImageFilePreview.h"
#include "kshowdebuggingoutput.h"


class KImg2mpgData : public KDialog
{
Q_OBJECT

public:

  KImg2mpgData(QWidget* parent = 0, const char * name = 0);
  virtual ~KImg2mpgData();
  void show();
  void ShowNumberImages( int Number );
  void closeEvent(QCloseEvent*);
  QPixmap LoadIcon( QString Name, int Group );
  void findExecutables( void );
  void RemoveTmpFiles( void );
      
public slots:

  void reset();
  void readStdout(KProcess *proc, char *buffer, int buflen);
  void readStderr(KProcess *proc, char *buffer, int buflen);
  void EncodeDone(KProcess* );
  void writeSettings();
  void readSettings();
  
  void slotMPEGFilenameDialog( void );
  void slotAudioFilenameDialog( void );
  void slotImagesFilesButtonAdd( void );
  void slotImagesFilesButtonDelete( void );
  void slotImagesFilesButtonUp( void );
  void slotImagesFilesButtonDown( void );
  void slotEncode( void );
  void slotQuit( void );
  void slotImagesFilesSelected( int Index );
  void SlotPortfolioDurationChanged ( int );

protected slots:

private:

  QString               VideoFormatConfig;
  QString               VideoTypeConfig;
  QString               ImageDurationConfig;
  QString               TransitionSpeedConfig;
  QColor                BackgroundColorConfig;
  QString               MPEGOutputFileConfig;
  QString               AudioInputFileConfig;
  KConfig*              config;
  QTime                 EncodingDuration, DurationTime;

  QString               NoneLabel;
  QString               img2mpg, montage, composite, convert, identify, ppmtoy4m, yuvscaler, mpeg2enc, mp2enc, mplex;
  bool                  Abort;
  bool                  Encoding;
  QString               EncodeString;
  QString               TemporaryFolderName;
  QString               FolderTemporaryIndex;
  QString               DebugOuputMessages;
  pid_t                 Img2mpgPidNum;
  KProcess*             Proc;
  QLabel*               ImageLabel;
  QLabel*               frame;
  QLabel*               label1;
  QLabel*               label3;
  QLabel*               label4;
  QLabel*               label5;
  QLabel*               label6;
  QLabel*               label7;
  QPushButton*          Encodebutton;
  KProgress*            progress;
  KHelpMenu*            helpMenu;
  QPushButton*          quitbutton;
  QPushButton*          helpbutton;
  QComboBox*            VideoTypeComboBox;
  QComboBox*            VideoFormatComboBox;
  QComboBox*            TransitionComboBox;
  QSpinBox*             DurationImageSpinBox;
  QGroupBox*            MPEGOutputFilename;
  KLineEdit*            MPEGOutputEDITFilename;
  QPushButton*          MPEGOutputBUTTONFilename;
  QGroupBox*            AudioInputFilename;
  KLineEdit*            AudioInputEDITFilename;
  QPushButton*          AudioInputBUTTONFilename;  
  KLineEdit*            TemporaryFolder;
  KIconLoader*          Icons;
  KColorButton*         BackgroundColorButton;
  QGroupBox*            ImagesFilesGroup;
  KListBox*             ImagesFilesListBox;              
  KButtonBox*           ImagesFilesButtonBox;
  QPushButton*          ImagesFilesButtonAdd;
  QPushButton*          ImagesFilesButtonDelete;
  QPushButton*          ImagesFilesButtonUp;
  QPushButton*          ImagesFilesButtonDown;
  QString               CommandLine;
  
  class ImageFilePreview*     PreviewImage;
  class KShowDebuggingOutput* DebuggingDialog;
            
protected:

};

#endif // KImg2mpgData_included
